@@ -50,6 +50,11 @@ sub const {
 	$this;
 }
 
+sub isConst {
+	my $this = shift;
+	exists($this->{const});
+}
+
 sub clone {
 	# deep copyはせずに、キー単位でのcopy-on-writeを行う。
 	my $this = shift;
@@ -907,6 +912,12 @@ Tripletail::Form オブジェクトを作成。
 
 このメソッドを呼び出すと、以後フォームデータの変更は不可能となる。
 
+=item isConst
+
+  $form->isConst
+
+フォームオブジェクトに対して const メソッドが呼ばれたかどうかを返す。
+
 =item clone
 
   $NewForm_obj = $form->clone
